@@ -23,7 +23,7 @@
 #include "webserver.h"
 #include "WebPages.h"
 
-#define VERSION 1.1
+#define VERSION 1.4
 
 extern char *getfqdn(char *);
 extern void trim(char *);
@@ -39,11 +39,10 @@ extern int splitobjective(char *, char *, char *, char *, int *, int *, int *);
 extern void trimallwhitespace(char *);
 extern int issyslog(char *);
 extern int get_syslog_dest(char *);
-
+extern void DebugMsg(const char *,...);
 
 // Make sure we return the size, or zero (for strings).
-int HandleWebPages(char *HTTPBuffer, char *HTTPOutputBuffer, int size,
-		   int http_listen_socket, int http_message_socket)
+int HandleWebPages(char *HTTPBuffer, char *HTTPOutputBuffer, int size, int http_listen_socket, int http_message_socket)
 {
 	int returncode = 0;
 	char *ArgPosition;
@@ -96,8 +95,7 @@ int HandleWebPages(char *HTTPBuffer, char *HTTPOutputBuffer, int size,
 			returncode +=
 			    Remote_Config(ArgPosition, pBuffer, psize);
 		} else if (!strncmp(HTTPBuffer, "/setremote", 10)) {
-			returncode +=
-			    Remote_Set(ArgPosition, pBuffer, psize);
+			returncode += Remote_Set(ArgPosition, pBuffer, psize);
 		} else if (!strncmp(HTTPBuffer, "/objective", 10)) {
 			returncode +=
 			    Objective_Config(ArgPosition, pBuffer, psize);
@@ -140,9 +138,9 @@ int HandleWebPages(char *HTTPBuffer, char *HTTPOutputBuffer, int size,
 
 int Status_Page(char *source, char *dest, int size)
 {
-	if(source && dest && size) {
-		snprintf(dest,size,
-			"<HTML><BODY><H1><CENTER>Welcome to SNARE for AIX version 1.1</H1><P>Please select from the menu on the left.</CENTER><p>\nStatus: <font color=green>Snare for AIX is currently running.</font>\n</BODY></HTML>");
+	if (source && dest && size) {
+		snprintf(dest, size,
+			"<HTML><BODY><H1><CENTER>Welcome to SNARE for AIX version 1.4</H1><P>Please select from the menu on the left.</CENTER><p>\nStatus: <font color=green>Snare for AIX is currently running.</font>\n</BODY></HTML>");
 	}
 	return (0);
 }
@@ -150,7 +148,7 @@ int Status_Page(char *source, char *dest, int size)
 int ShowUsers(int http_socket)
 {
 	struct passwd *user;
-	int retval=0;
+	int retval = 0;
 	char temp[1024];
 	char hostid[MAX_HOSTID];
 	char *sec_char;
@@ -278,7 +276,7 @@ int ShowUsers(int http_socket)
 int ShowGroups(int http_socket)
 {
 	struct group *groupp;
-	int retval=0;
+	int retval = 0;
 	char temp[1024];
 	char hostid[MAX_HOSTID];
 
@@ -302,7 +300,7 @@ int ShowGroupMembers(int http_socket)
 {
 	struct group *groupp;
 	struct passwd *userp;
-	int retval=0;
+	int retval = 0;
 	char temp[1024];
 	int firstmember = 1;
 	char hostid[MAX_HOSTID];
@@ -313,10 +311,10 @@ int ShowGroupMembers(int http_socket)
 		struct UNode *next;
 	};
 
-	struct UNode *head=NULL;
-	struct UNode *current=NULL;
-	struct UNode *tail=NULL;
-	int found=0;
+	struct UNode *head = NULL;
+	struct UNode *current = NULL;
+	struct UNode *tail = NULL;
+	int found = 0;
 
 	getfqdn(hostid);
 
@@ -330,19 +328,19 @@ int ShowGroupMembers(int http_socket)
 	// Linked list would be a bit of an an overkill.. but I need to use one.
 	setpwent();
 	// Look through the password file for users with this pgid
-	while((userp = (struct passwd *) getpwent()) != (struct passwd *) NULL) {
+	while ((userp = (struct passwd *) getpwent()) != (struct passwd *) NULL) {
 		// Zap up a simple linked list
-		tail=malloc(sizeof(struct UNode));
-		strncpy(tail->username,userp->pw_name,sizeof(tail->username));
-		tail->gid=userp->pw_gid;
+		tail = malloc(sizeof(struct UNode));
+		strncpy(tail->username, userp->pw_name, sizeof(tail->username));
+		tail->gid = userp->pw_gid;
 		tail->next = NULL;
-		if(!head) {
-			head=tail;
+		if (!head) {
+			head = tail;
 		}
-		if(current) {
-			current->next=tail;
+		if (current) {
+			current->next = tail;
 		}
-		current=tail;
+		current = tail;
 	}
 	endpwent();
 
@@ -350,42 +348,42 @@ int ShowGroupMembers(int http_socket)
 	while ((groupp = (struct group *) getgrent()) != (struct group *) NULL) {
 		firstmember = 1;
 		snprintf(temp, sizeof (temp), "%s\t%d\t", groupp->gr_name,
-			 groupp->gr_gid);
+			groupp->gr_gid);
 		retval += send(http_socket, temp, strlen(temp), 0);
 
 		// Push out our linke list users first
-		current=head;
-		while(current) {
-			if(current->gid == groupp->gr_gid) {
-				if(firstmember) {
-					firstmember=0;
+		current = head;
+		while (current) {
+			if (current->gid == groupp->gr_gid) {
+				if (firstmember) {
+					firstmember = 0;
 				} else {
-					retval+=send(http_socket,",",1,0);
+					retval += send(http_socket, ",", 1, 0);
 				}
-				retval+=send(http_socket,current->username,strlen(current->username),0);
+				retval += send(http_socket,current->username,strlen(current->username),0);
 			}
-			current=current->next;
+			current = current->next;
 		}
 
 		while (*(groupp->gr_mem) != NULL) {
 			// Scan through our linked list to avoid duplicates.
-			found=0;
-			current=head;
-			while(current) {
-				if(current->gid == groupp->gr_gid && strcmp(*(groupp->gr_mem), current->username) == 0) {
+			found = 0;
+			current = head;
+			while (current) {
+				if (current->gid == groupp->gr_gid && strcmp(*(groupp->gr_mem), current->username) == 0) {
 					// Already printed.
-					found=1;
+					found = 1;
 				}
-				current=current->next;
+				current = current->next;
 			}
-			if(!found) {
+			if (!found) {
 				// New user
-				if(firstmember) {
-					firstmember=0;
+				if (firstmember) {
+					firstmember = 0;
 				} else {
-					retval+=send(http_socket,",",1,0);
+					retval += send(http_socket,",",1,0);
 				}
-				retval+=send(http_socket,*(groupp->gr_mem),strlen(*(groupp->gr_mem)),0);
+				retval += send(http_socket,*(groupp->gr_mem),strlen(*(groupp->gr_mem)),0);
 			}
 			groupp->gr_mem++;
 		}
@@ -395,9 +393,9 @@ int ShowGroupMembers(int http_socket)
 
 	// Clear our temporary linked list.
 	while(head) {
-		current=head->next;
+		current = head->next;
 		free(head);
-		head=current;
+		head = current;
 	}
 
 	return (1);
@@ -406,22 +404,29 @@ int ShowGroupMembers(int http_socket)
 int Network_Config(char *source, char *dest, int size)
 {
 	struct Reg_Config config_struct;
+	struct Reg_Host host_struct;
 	int dw_config_error;
 	char str_DestPort[10];
+	char str_net_count[10];
 	char str_conferr[10], str_neterr[10];
 	int SyslogPriority, SyslogFacility;
 	char *str_facility[] = {"Kernel","User","Mail","Daemon","Auth","Syslog","Lpr","News","UUCP","Cron","Authpriv","Ftp","Local0","Local1","Local2","Local3","Local4","Local5","Local6","Local7"};
 	char *str_priority[] = {"Emergency","Alert","Critical","Error","Warning","Notice","Information","Debug"};
 	int i;
+	int i_network_count= 1;
 
-	if(!source || !dest || !size) {
+	FILE *configfile = (FILE *) NULL;
+
+	if (!source || !dest || !size) {
 		return(0);
 	}
 
 	strcpy(config_struct.str_ClientName, "");
 	strcpy(config_struct.str_FileDestination, "");
-	strcpy(config_struct.str_NetworkDestination, "");
+	strcpy(host_struct.str_NetworkDestination, "");
 	config_struct.dw_stdout = 0;
+	host_struct.dw_DestPort = 0;
+	host_struct.dw_Protocol = 0;
 
 	dw_config_error = Read_Config_From_File(&config_struct);
 
@@ -454,26 +459,37 @@ int Network_Config(char *source, char *dest, int size)
 	strncat(dest, config_struct.str_ClientName, size - strlen(dest));
 	strncat(dest, "\"></td></tr>", size - strlen(dest));
 
-	strncat(dest,
-		"<tr bgcolor=#FFFFBB><td>Destination Server address </td><td><input type=text name=str_NetworkDestination size=25 value=\"",
-		size - strlen(dest));
-	strncat(dest, config_struct.str_NetworkDestination,
-		size - strlen(dest));
-	strncat(dest, "\"></td></tr>", size - strlen(dest));
+	configfile = Find_First_Network();
+	if (configfile) {
+		//Grab all of the Network details, we only want the last one
+		while (Get_Next_Network(configfile, &host_struct));
+		strncat(dest,
+			"<tr bgcolor=#FFFFCC><td colspan=2><strong>Destination</strong></td></tr>", size - strlen(dest));
+		strncat(dest,
+			"<tr bgcolor=#FFFF88><td>Destination Server address </td><td><input type=text name=str_NetworkDestination size=25 value=\"",
+			size - strlen(dest));
+		strncat(dest, host_struct.str_NetworkDestination,
+			size - strlen(dest));
+		strncat(dest, "\"></td></tr>", size - strlen(dest));
 
-	strncat(dest,
-		"<tr bgcolor=#FFFFCC><td>Destination UDP Port (514 for Syslog)</td><td><input type=text name=dw_DestPort size=8 value=\"",
-		size - strlen(dest));
+		strncat(dest,
+			"<tr bgcolor=#FFFF88><td>Destination Port (514 for syslog)</td><td><input type=text name=dw_DestPort size=8 value=\"",
+			size - strlen(dest));
 
-	if (config_struct.dw_DestPort) {
-		snprintf(str_DestPort, 10, "%d", config_struct.dw_DestPort);
+		if (host_struct.dw_DestPort) {
+			snprintf(str_DestPort, 10, "%d", host_struct.dw_DestPort);
+		} else {
+			str_DestPort[0] = '\0';
+		}
+
+		strncat(dest, str_DestPort, size - strlen(dest));
+		strncat(dest, "\"></td></tr>", size - strlen(dest));
 	} else {
-		str_DestPort[0] = '\0';
+		strncat(dest,
+			"<tr bgcolor=#FFFF88><td colspan=2>ERROR</td></tr>",
+			size - strlen(dest));
 	}
 
-	strncat(dest, str_DestPort, size - strlen(dest));
-	strncat(dest, "\"></td></tr>", size - strlen(dest));
-
 	strncat(dest,
 		"<tr bgcolor=#FFFFBB><td>Destination FileName </td><td><input type=text name=str_FileDestination size=25 value=\"",
 		size - strlen(dest));
@@ -490,35 +506,31 @@ int Network_Config(char *source, char *dest, int size)
 
 	SyslogPriority = config_struct.dw_Syslog & 7;
 	SyslogFacility = config_struct.dw_Syslog >> 3;
-	if(SyslogFacility > 11) {
+	if (SyslogFacility > 11) {
 		SyslogFacility -= 4;
 	}
 
-	strncat(dest,"<tr bgcolor=#FFFFCC><td>SYSLOG Facility (optional) </td><td><select name=SyslogFacility>",size - strlen(dest));
-        for (i = 0; i < 19; i++)
-        {
-                strncat(dest,"<option",size - strlen(dest));
-                if (i == SyslogFacility)
-                        strncat(dest," selected>",size - strlen(dest));
-                else
-                        strncat(dest,">",size - strlen(dest));
-                strncat(dest,str_facility[i],size - strlen(dest));
-        }
-	strncat(dest,"</select></td></tr>",size - strlen(dest));
-
-	strncat(dest,"<tr bgcolor=#FFFFBB><td>SYSLOG Priority (optional) </td><td><select name=SyslogPriority>",size - strlen(dest));
-        for (i = 0; i < 8; i++)
-        {
-                strncat(dest,"<option",size - strlen(dest));
-                if (i == SyslogPriority)
-                        strncat(dest," selected>",size - strlen(dest));
-                else
-                        strncat(dest,">",size - strlen(dest));
-                strncat(dest,str_priority[i],size - strlen(dest));
-        }
-        strncat(dest,"</select></td></tr>",size - strlen(dest));
-
+	strncat(dest, "<tr bgcolor=#FFFFCC><td>SYSLOG Facility (optional) </td><td><select name=SyslogFacility>",size - strlen(dest));
+	for (i = 0; i < 19; i++) {
+		strncat(dest, "<option", size - strlen(dest));
+		if (i == SyslogFacility)
+			strncat(dest, " selected>", size - strlen(dest));
+		else
+			strncat(dest, ">", size - strlen(dest));
+		strncat(dest, str_facility[i], size - strlen(dest));
+	}
+	strncat(dest, "</select></td></tr>", size - strlen(dest));
 
+	strncat(dest, "<tr bgcolor=#FFFFBB><td>SYSLOG Priority (optional) </td><td><select name=SyslogPriority>",size - strlen(dest));
+	for (i = 0; i < 8; i++) {
+		strncat(dest, "<option",size - strlen(dest));
+		if (i == SyslogPriority)
+			strncat(dest, " selected>",size - strlen(dest));
+		else
+			strncat(dest, ">",size - strlen(dest));
+		strncat(dest, str_priority[i],size - strlen(dest));
+	}
+	strncat(dest, "</select></td></tr>",size - strlen(dest));
 
 	strncat(dest, "</table><br>", size - strlen(dest));
 	strncat(dest, "<input type=submit value=\"Change Configuration\">    ",
@@ -532,28 +544,29 @@ int Network_Config(char *source, char *dest, int size)
 int Network_Set(char *source, char *dest, int size)
 {
 	char *psource = source;
-	char Variable[100], web_port[100],syslog_fac[100],syslog_pri[100];
+	char Variable[100], web_port[100], syslog_fac[100], syslog_pri[100];
 	char Argument[100];
 	struct Reg_Config config_struct;
-	int dw_error_config = 0,i=0;
+	struct Reg_Host * RHead=NULL;
+	struct Reg_Host * RCurrent=NULL;
+	struct Reg_Host * RTail=NULL;
+	int dw_error_config = 0, i = 0;
 	char *str_facility[] = {"Kernel","User","Mail","Daemon","Auth","Syslog","Lpr","News","UUCP","Cron","Authpriv","Ftp","Local0","Local1","Local2","Local3","Local4","Local5","Local6","Local7"};
         char *str_priority[] = {"Emergency","Alert","Critical","Error","Warning","Notice","Information","Debug"};
 	int dw_SyslogClass = 0;
+	int dw_error_port=0;
 
 	// This function will display that the remote audit configurations have been changed, or there have been errors
 	strncpy(dest, "<h1><center>SNARE Configuration</h1>", size);
 
 	// Note that all the possible variables do NOT have to be in the URL. The ones that are selected
 	// via a checkbox will not be listed if the checkbox has been deselected.
-	// Checking is limited to ensuring the Detsination port in the range 1-65535.
+	// Checking is limited to ensuring the Destination port in the range 1-65535.
 	// The variable associated with the checkbox (dw_Syslog) must be
 	// exactly "on" or it will be defaulted to "off".
 	// str_Destination,str_Delimiter_Text,str_ClientName can be anything it wants to be, so long 
 	// as it is within size bounds.
 
-	config_struct.dw_DestPort = -1;
-	strncpy(config_struct.str_NetworkDestination, "",
-		sizeof (config_struct.str_NetworkDestination));
 	strncpy(config_struct.str_ClientName, "",
 		sizeof (config_struct.str_ClientName));
 	strncpy(web_port, "", sizeof (web_port));
@@ -571,12 +584,25 @@ int Network_Set(char *source, char *dest, int size)
 		}
 
 		if (strstr(Variable, "str_NetworkDestination") != NULL) {
-			strncpy(config_struct.str_NetworkDestination, Argument,
-				sizeof (config_struct.str_NetworkDestination));
+			RCurrent = (struct Reg_Host *)malloc(sizeof(struct Reg_Host));
+			RCurrent->next=NULL;
+			RCurrent->dw_DestPort = 0;
+			RCurrent->dw_Protocol = SOCKETTYPE_UDP;
+			if (RCurrent) {
+				strncpy(RCurrent->str_NetworkDestination, Argument,
+					sizeof (RCurrent->str_NetworkDestination));
+			}
 		}
 
 		if (strstr(Variable, "dw_DestPort") != NULL) {
 			strncpy(web_port, Argument, sizeof (web_port));
+			if (RCurrent) {
+				RCurrent->dw_DestPort = atoi(web_port);
+			}
+			if (strcmp(RCurrent->str_NetworkDestination, "") != 0) {
+				RHead=RCurrent;
+				RTail=RCurrent;
+			}
 		}
 
 		if (strstr(Variable, "str_FileDestination") != NULL) {
@@ -590,36 +616,42 @@ int Network_Set(char *source, char *dest, int size)
 				config_struct.dw_stdout = 0;
 			}
 		}
-		if (strstr(Variable,"SyslogPriority") != NULL) {
-                        strncpy(syslog_pri,Argument,sizeof(syslog_pri));
-                }
-                if (strstr(Variable,"SyslogFacility") != NULL) {
-                        strncpy(syslog_fac,Argument,sizeof(syslog_fac));
-                }
+		if (strstr(Variable, "SyslogPriority") != NULL) {
+			strncpy(syslog_pri, Argument, sizeof(syslog_pri));
+		}
+		if (strstr(Variable, "SyslogFacility") != NULL) {
+			strncpy(syslog_fac, Argument, sizeof(syslog_fac));
+		}
 	}
 
 	for (i = 0; i < 8; i++) {
-                if (strstr(syslog_pri,str_priority[i]) != NULL)
-                        config_struct.dw_Syslog = i;
-        }
-        for (i = 0; i < 19; i++)
-        {
-                if (strstr(syslog_fac,str_facility[i]) != NULL)
-                        dw_SyslogClass = i;
-        }
-	if(dw_SyslogClass > 11) {
-                dw_SyslogClass = dw_SyslogClass + 4;
-        }
-        config_struct.dw_Syslog = config_struct.dw_Syslog | (dw_SyslogClass << 3);
+		if (strstr(syslog_pri, str_priority[i]) != NULL)
+			config_struct.dw_Syslog = i;
+	}
+	for (i = 0; i < 19; i++) {
+		if (strstr(syslog_fac, str_facility[i]) != NULL)
+			dw_SyslogClass = i;
+	}
+	if (dw_SyslogClass > 11) {
+		dw_SyslogClass = dw_SyslogClass + 4;
+	}
+	config_struct.dw_Syslog = config_struct.dw_Syslog | (dw_SyslogClass << 3);
 
-	config_struct.dw_DestPort = atoi(web_port);
-	if (strlen(config_struct.str_NetworkDestination)
-	    && ((config_struct.dw_DestPort < 1)
-	    || (config_struct.dw_DestPort > 65535))) {
-		strncat(dest,
-			"The Destination Port value must be between 1 and 65535. Use the 'back' button to change the value.",
-			size - strlen(dest));
-	} else {
+	// Check that all the port numbers are valid
+	RCurrent = RHead;
+	while (RCurrent) {
+		if (strlen(RCurrent->str_NetworkDestination)
+		    && ((RCurrent->dw_DestPort < 1)
+		    || (RCurrent->dw_DestPort > 65535))) {
+			strncat(dest,
+				"The Destination Port value must be between 1 and 65535. Use the 'back' button to change the value.",
+				size - strlen(dest));
+			dw_error_port++;
+			break;
+		}	
+		RCurrent = RCurrent->next;
+	}
+	if (!dw_error_port) {
 		void *rampointer = (void *) NULL;
 		char *position;
 		char inputbuffer[MAX_AUDIT_CONFIG_LINE];
@@ -685,29 +717,25 @@ int Network_Set(char *source, char *dest, int size)
 								config_struct.
 								str_FileDestination);
 						}
-						if (strlen
-						    (config_struct.
-						     str_NetworkDestination)
-						    && config_struct.
-						    dw_DestPort) {
+						RCurrent = RHead;
+						while (RCurrent) {
 							fprintf(configfile,
 								"	network=%s:%d\n",
-								config_struct.
+								RCurrent->
 								str_NetworkDestination,
-								config_struct.
+								RCurrent->
 								dw_DestPort);
-
+							RCurrent=RCurrent->next;
 						}
 						if (config_struct.dw_stdout) {
 							fprintf(configfile,
 								"	file=stdout\n");
 						}
-						if(config_struct.dw_Syslog) {
+						if (config_struct.dw_Syslog) {
 							fprintf(configfile,
-								"	syslog=%d\n",config_struct.dw_Syslog);
+								"	syslog=%d\n", config_struct.dw_Syslog);
 						}
-						fprintf(configfile,"\n");
-
+						fprintf(configfile, "\n");
 						skip = 1;
 					} else {
 						skip = 0;
@@ -741,13 +769,15 @@ int Network_Set(char *source, char *dest, int size)
 						config_struct.
 						str_FileDestination);
 				}
-				if (strlen(config_struct.str_NetworkDestination)
-				    && config_struct.dw_DestPort) {
+				RCurrent = RHead;
+				while (RCurrent) {
 					fprintf(configfile,
 						"	network=%s:%d\n",
-						config_struct.
+						RCurrent->
 						str_NetworkDestination,
-						config_struct.dw_DestPort);
+						RCurrent->
+						dw_DestPort);
+					RCurrent=RCurrent->next;
 				}
 				if (config_struct.dw_stdout) {
 					fprintf(configfile, "	file=stdout\n");
@@ -787,7 +817,7 @@ int Remote_Config(char *source, char *dest, int size)
 	char str_WebPort[10];
 	char str_remerr[10];
 
-	if(!source || !dest || !size) {
+	if (!source || !dest || !size) {
 		return(0);
 	}
 
@@ -851,7 +881,7 @@ int Remote_Config(char *source, char *dest, int size)
 		"<tr bgcolor=#FFFFCC><td>Password to allow remote control of SNARE </td><td><input type=password name=str_Password size=12 value=\"",
 		size - strlen(dest));
 	strncat(dest, remote_struct.str_Password, size - strlen(dest));
-	strncat(dest, "\"><input type=hidden name=str_OldPassword value=\"",size -strlen(dest));
+	strncat(dest, "\"><input type=hidden name=str_OldPassword value=\"", size - strlen(dest));
 	strncat(dest, remote_struct.str_Password, size - strlen(dest));
 	strncat(dest, "\"></td></tr>", size - strlen(dest));
 
@@ -876,7 +906,7 @@ int Remote_Set(char *source, char *dest, int size)
 	char *psource = source;
 	char Variable[100], web_port[100];
 	char Argument[100];
-	char OldPassword[100]="";
+	char OldPassword[100] = "";
 	struct Reg_Remote remote_struct;
 	int dw_error = 0;
 
@@ -912,15 +942,14 @@ int Remote_Set(char *source, char *dest, int size)
 				sizeof (remote_struct.str_RestrictIP));
 		}
 		if (strstr(Variable, "str_Password") != NULL) {
-			if(strcmp(Argument,remote_struct.str_Password)!=0) {
-				strncpy(remote_struct.str_Password,Argument,sizeof(remote_struct.str_Password));
+			if (strcmp(Argument, remote_struct.str_Password) != 0) {
+				strncpy(remote_struct.str_Password, Argument, sizeof(remote_struct.str_Password));
 
 			}
 		}
 
 		if (strstr(Variable, "str_OldPassword") != NULL) {
-			strncpy(OldPassword, Argument,
-				sizeof (OldPassword));
+			strncpy(OldPassword, Argument, sizeof(OldPassword));
 		}
 		if (strstr(Variable, "dw_Allow") != NULL) {
 			if (strcmp(Argument, "on") == 0)
@@ -950,11 +979,11 @@ int Remote_Set(char *source, char *dest, int size)
 			"The Web Port value must be between 1 and 65535. Use the 'back' button to change the value.",
 			size - strlen(dest));
 	} else {
-		void * rampointer = (void *) NULL;
-		char * position;
+		void *rampointer = (void *)NULL;
+		char *position;
 		char inputbuffer[MAX_AUDIT_CONFIG_LINE];
 		int headertype = 0;
-		FILE * configfile=(FILE *)NULL;
+		FILE *configfile = (FILE *)NULL;
 
 		rampointer = Load_Config_File();
 
@@ -1019,11 +1048,12 @@ int Remote_Set(char *source, char *dest, int size)
 								remote_struct.
 								str_RestrictIP);
 						}
+						// Reject passwords with backslashes.
 						if (strlen
 						    (remote_struct.str_Password)
 						    && remote_struct.
 						    dw_Password) {
-							if(strstr(remote_struct.str_Password,"\\")) {
+							if (strstr(remote_struct.str_Password,"\\")) {
 								strncat(dest,
 									"Sorry, passwords with the backslash character are not allowed.<br>Please use the Back Button, and try another password.<br> The other ",
 									size - strlen(dest));
@@ -1086,7 +1116,7 @@ int Remote_Set(char *source, char *dest, int size)
 				if (strlen(remote_struct.str_Password)
 				    && remote_struct.dw_Password) {
 					fprintf(configfile, "	accesskey=%s\n",
-						(char *)crypt(remote_struct.str_Password,SALT));
+						(char *)crypt(remote_struct.str_Password, SALT));
 				}
 				fprintf(configfile, "\n");
 			}
@@ -1128,7 +1158,7 @@ int Objective_Config(char *source, char *dest, int size)
 
 	FILE *configfile = (FILE *) NULL;
 
-	if(!source || !dest || !size) {
+	if (!source || !dest || !size) {
 		return(0);
 	}
 
@@ -1311,17 +1341,13 @@ int Objective_Display(char *source, char *dest, int size)
 				sizeof (reg_objective.str_critic));
 			strncpy(reg_objective.str_eventid_match, LOGON_LOGOFF_EVENTS,
 				sizeof (reg_objective.str_eventid_match));
-			strncpy(reg_objective.str_general_match,
-				".*",
+			strncpy(reg_objective.str_general_match, ".*",
 				sizeof (reg_objective.str_general_match));
-			strncpy(reg_objective.str_general_match_type,
-				"Any",
+			strncpy(reg_objective.str_general_match_type, "Any",
 				sizeof (reg_objective.str_general_match_type));
-			strncpy(reg_objective.str_user_match_type,
-				"Any",
+			strncpy(reg_objective.str_user_match_type, "Any",
 				sizeof (reg_objective.str_user_match_type));
-			strncpy(reg_objective.str_user_match,
-				".*",
+			strncpy(reg_objective.str_user_match, ".*",
 				sizeof (reg_objective.str_user_match));
 		}
 
@@ -1363,8 +1389,7 @@ int Objective_Display(char *source, char *dest, int size)
 			size - strlen(dest));
 		strncat(dest, FILE_READ_TOKEN, size - strlen(dest));
 		if (strcmp
-		    (reg_objective.str_eventid_match,
-		     FILE_READ_EVENTS) == 0) {
+		    (reg_objective.str_eventid_match, FILE_READ_EVENTS) == 0) {
 			strncat(dest, " checked", size - strlen(dest));
 			i_set = 1;
 		}
@@ -1373,8 +1398,7 @@ int Objective_Display(char *source, char *dest, int size)
 			size - strlen(dest));
 		strncat(dest, FILE_WRITE_TOKEN, size - strlen(dest));
 		if (strcmp
-		    (reg_objective.str_eventid_match,
-		     FILE_WRITE_EVENTS) == 0) {
+		    (reg_objective.str_eventid_match, FILE_WRITE_EVENTS) == 0) {
 			strncat(dest, " checked", size - strlen(dest));
 			i_set = 1;
 		}
@@ -1401,8 +1425,7 @@ int Objective_Display(char *source, char *dest, int size)
 			">Start or stop program execution  </td><td><input type=radio name=str_eventid_match value=",
 			size - strlen(dest));
 		strncat(dest, ADMIN_TOKEN, size - strlen(dest));
-		if (strcmp(reg_objective.str_eventid_match, ADMIN_EVENTS) ==
-		    0) {
+		if (strcmp(reg_objective.str_eventid_match, ADMIN_EVENTS) == 0) {
 			strncat(dest, " checked", size - strlen(dest));
 			i_set = 1;
 		}
@@ -1432,8 +1455,7 @@ int Objective_Display(char *source, char *dest, int size)
 			"<input type=radio name=str_general_match_type value=Any",
 			size - strlen(dest));
 
-		if (strcmp(reg_objective.str_general_match, ".*") ==
-		    0) {
+		if (strcmp(reg_objective.str_general_match, ".*") == 0) {
 			strncat(dest, " checked", size - strlen(dest));
 			strncpy(reg_objective.str_general_match_type,"Any",sizeof(reg_objective.str_general_match_type));
 		}
@@ -1454,7 +1476,6 @@ int Objective_Display(char *source, char *dest, int size)
 			strncat(dest, " checked", size - strlen(dest));
 		strncat(dest, ">Exclude    </td></tr>", size - strlen(dest));
 
-
 		strncat(dest,
 			"<tr bgcolor=#FFFFBB><td>Search Term<br><i>(regular expression)</i></td><td><input type=text name=str_general_match size=50 value=\"",
 			size - strlen(dest));
@@ -1717,7 +1738,6 @@ void *Load_Config_File()
 		return ((void *) NULL);
 	}
 
-
 	location = malloc(filesize + 1);
 
 	if (!location) {
@@ -1882,23 +1902,19 @@ int Objective_Result(char *source, char *dest, int size)
 			strncpy(reg_objective.str_eventid_match,
 				LOGON_LOGOFF_EVENTS,
 				sizeof(reg_objective.str_eventid_match));
-		} else if (strstr(str_eventid_radio, FILE_READ_TOKEN) !=
-			   NULL) {
+		} else if (strstr(str_eventid_radio, FILE_READ_TOKEN) != NULL) {
 			strncpy(reg_objective.str_eventid_match,
 				FILE_READ_EVENTS,
 				sizeof(reg_objective.str_eventid_match));
-		} else if (strstr(str_eventid_radio, FILE_WRITE_TOKEN) !=
-			   NULL) {
+		} else if (strstr(str_eventid_radio, FILE_WRITE_TOKEN) != NULL) {
 			strncpy(reg_objective.str_eventid_match,
 				FILE_WRITE_EVENTS,
 				sizeof(reg_objective.str_eventid_match));
-		} else if (strstr(str_eventid_radio, FILE_ATTRIB_TOKEN) !=
-			   NULL) {
+		} else if (strstr(str_eventid_radio, FILE_ATTRIB_TOKEN) != NULL) {
 			strncpy(reg_objective.str_eventid_match,
 				FILE_ATTRIB_EVENTS,
 				sizeof(reg_objective.str_eventid_match));
-		} else if (strstr(str_eventid_radio, PROCESS_TOKEN) !=
-			   NULL) {
+		} else if (strstr(str_eventid_radio, PROCESS_TOKEN) != NULL) {
 			strncpy(reg_objective.str_eventid_match,
 				PROCESS_EVENTS,
 				sizeof(reg_objective.str_eventid_match));
@@ -2338,7 +2354,7 @@ int Objective_Result(char *source, char *dest, int size)
 int DefaultHeader(char *source, char *dest, int size)
 {
 
-	if(!source || !dest || !size) {
+	if (!source || !dest || !size) {
 		return(0);
 	}
 
@@ -2423,7 +2439,7 @@ int DefaultHeader(char *source, char *dest, int size)
 int DefaultFooter(char *source, char *dest, int size)
 {
 
-	if(!source || !dest || !size) {
+	if (!source || !dest || !size) {
 		return(0);
 	}
 
@@ -2470,7 +2486,7 @@ int Restart(char *source, char *dest, int size, int socketone, int sockettwo)
 {
 	int pid;
 
-	if(!source || !dest || !size) {
+	if (!source || !dest || !size) {
 		return(0);
 	}
 
@@ -2540,12 +2556,12 @@ int InterSectImage(char *source, char *dest, int size)
 		"OVTfBJId7ugvoxJBm0qUZ66IdqhtbKNUdCoq0ehUCl6ZsWrbKKodh1x0kFAliFQL8NG0lm5L"
 		"CJAe3QICADs=", 2029);
 
-	size2=base64decode (temp, IntersectGif);
-	if(size2>size) {
+	size2 = base64decode(temp, IntersectGif);
+	if (size2 > size) {
 		return(0);
 	}
 
-	memcpy(dest,temp,size2);
+	memcpy(dest, temp, size2);
 	return(size2);
 }
 
@@ -2631,10 +2647,10 @@ int Read_Config_From_File(struct Reg_Config *config_struct)
 		return (0);
 
 	strcpy(config_struct->str_ClientName, "");
-	strcpy(config_struct->str_NetworkDestination, "");
+	//strcpy(config_struct->str_NetworkDestination, "");
 	strcpy(config_struct->str_FileDestination, "");
 	config_struct->dw_stdout = 0;
-	config_struct->dw_DestPort = 0;
+	//config_struct->dw_DestPort = 0;
 
 	while (fgets(inputbuffer, MAX_AUDIT_CONFIG_LINE, configfile)) {
 		// Kill whitespace from start and end of line.
@@ -2673,21 +2689,11 @@ int Read_Config_From_File(struct Reg_Config *config_struct)
 								SIZE_OF_DESTINATION);
 						}
 					} else if (isnetwork(inputbuffer)) {
-						config_struct->dw_DestPort =
-						    getnetwork(inputbuffer,
-							       config_struct->
-							       str_NetworkDestination,
-							       SIZE_OF_DESTINATION);
-						if (!config_struct->dw_DestPort) {
-							strncpy(config_struct->
-								str_NetworkDestination,
-								"",
-								SIZE_OF_DESTINATION);
-						}
+						// Ignore network entries for now, we will come back to them later
 					} else if (issyslog(inputbuffer)) {
-						config_struct->dw_Syslog=get_syslog_dest(inputbuffer);
-						if(!config_struct->dw_Syslog) {
-							config_struct->dw_Syslog=13;
+						config_struct->dw_Syslog = get_syslog_dest(inputbuffer);
+						if (!config_struct->dw_Syslog) {
+							config_struct->dw_Syslog = 13;
 						}
 					} else if (isstdout(inputbuffer)) {
 						config_struct->dw_stdout = 1;
@@ -2790,13 +2796,14 @@ int Read_Remote_From_File(struct Reg_Remote *remote_struct)
 }
 
 // Return the host identifier
-int getnetwork(char *string, char *host, int length)
+int getnetwork(char *string, char *host, int length, int *protocol)
 {
 	char *stringp = string;
-	char *pos;
+	char *pos, *pos2;
 	char strPort[10];
 
 	stringp = strstr(string, "=");
+	*protocol=SOCKETTYPE_UDP;
 
 	if (stringp != (char *) NULL) {
 		stringp++;
@@ -2814,7 +2821,6 @@ int getnetwork(char *string, char *host, int length)
 	} else {
 		return (0);
 	}
-	return(0);
 }
 
 int iswebfilename(char *string)
@@ -2839,6 +2845,36 @@ int isstdout(char *string)
 	}
 }
 
+FILE *Find_First_Network()
+{
+	FILE *configfile;
+	char inputbuffer[MAX_AUDIT_CONFIG_LINE];
+	int headertype = 0;
+
+	configfile = fopen(CONFIG_FILENAME, "r");
+	if (!configfile)
+		return (configfile);
+
+	while (fgets(inputbuffer, MAX_AUDIT_CONFIG_LINE, configfile)) {
+		// Kill whitespace from start and end of line.
+		trim(inputbuffer);
+
+		if (!iscomment(inputbuffer)) {
+			// Is this line a header?
+			if (isheader(inputbuffer)) {
+				headertype = getheader(inputbuffer);
+				if (headertype == CONFIG_OUTPUT) {
+					return (configfile);
+				}
+			}
+		}
+	}
+	// If we have made it this far, then we don't have any objectives.
+	fclose(configfile);
+	configfile = (FILE *) NULL;
+	return (configfile);
+}
+
 FILE *Find_First_Objective()
 {
 	FILE *configfile;
@@ -2869,6 +2905,40 @@ FILE *Find_First_Objective()
 	return (configfile);
 }
 
+int Get_Next_Network(FILE * configfile, struct Reg_Host *host_struct) {
+	char inputbuffer[MAX_AUDIT_CONFIG_LINE];
+	while (fgets(inputbuffer, MAX_AUDIT_CONFIG_LINE, configfile)) {
+		trim(inputbuffer);
+
+		if (isheader(inputbuffer)) {
+			return (0);
+		}
+
+		if(strlen(inputbuffer) < 3) {
+			continue;
+		}
+
+		if (isnetwork(inputbuffer)) {
+			host_struct->dw_DestPort =
+			    getnetwork(inputbuffer,
+				       host_struct->
+				       str_NetworkDestination,
+				       SIZE_OF_DESTINATION,
+				       &host_struct->dw_Protocol);
+			if (!host_struct->dw_DestPort) {
+				// Give it the default value
+				host_struct->dw_DestPort = 6161;
+//				strncpy(host_struct->
+//					str_NetworkDestination,
+//					"",
+//					SIZE_OF_DESTINATION);
+			}
+			return(1);
+		}
+	}
+	return (0);
+}
+
 int Get_Next_Objective(FILE * configfile, struct Reg_Objective *objective)
 {
 	char inputbuffer[MAX_AUDIT_CONFIG_LINE];
@@ -2889,7 +2959,7 @@ int Get_Next_Objective(FILE * configfile, struct Reg_Objective *objective)
 			return (0);
 		}
 
-		if(strlen(inputbuffer) < 3) {
+		if (strlen(inputbuffer) < 3) {
 			continue;
 		}
 
@@ -2930,7 +3000,7 @@ int Get_Next_Objective(FILE * configfile, struct Reg_Objective *objective)
 
 			// Strip out our ^( and )$ characters
 			// .. and zap in commas
-			if(user[0]=='^' && user[1]=='(' && strlen(user) >= 4) {
+			if (user[0]=='^' && user[1]=='(' && strlen(user) >= 4) {
 				int count;
 				int userlen=strlen(user);
 				if(user[userlen-2]==')' && user[userlen-1]=='$') {
