@@ -1,6 +1,6 @@
 /**********************************************************
  *
- * Snare for AIX version 1.1
+ * Snare for AIX version 1.4
  *
  * Author: InterSect Alliance Pty Ltd
  *
@@ -26,6 +26,7 @@
  * History:
  *       7/10/2004  Test version - based on Snare for AIX
  *       27/10/2005 Version 1.1 released 
+ *       2/8/2006   Latest updates to fix syslog truncation
  *
  **********************************************************
  *
@@ -75,7 +76,7 @@ void DebugMsg(const char *pszFormat, ...);
 // Linked List Functions
 static Node *head, *currentnode;
 static int AuditDestination = AUDIT_TO_STDOUT;	// Send to STDOUT unless otherwise stated.
-static int SyslogDestination=13;			// user.notice
+static int SyslogDestination = 13;			// user.notice
 
 char hostid[MAX_HOSTID] = "";
 int remote_allow = 0;
@@ -84,8 +85,21 @@ char remote_restrictip[16] = "";
 char remote_password[256] = "";
 
 FILE *AuditFile = (FILE *) NULL;
-int AuditSocket = 0;
-struct sockaddr_in AuditSocketName;
+//int AuditSocket = 0;
+//struct sockaddr_in AuditSocketName;
+
+int MCCount=0;
+MsgCache *MCHead=NULL;
+MsgCache *MCTail=NULL;
+MsgCache *MCCurrent=NULL;
+
+HostNode *hhead=NULL;
+HostNode *htail=NULL;
+HostNode *hcurrent=NULL;
+
+int SocketType = SOCKETTYPE_UDP;
+
+int HNodeCounter=0;
 
 /* Define an error variable so we can communicate with auditsvc */
 extern int errno;
@@ -101,27 +115,27 @@ int main(int argc, char *argv[])
 	// struct in_addr ia;
 
 	/* Audit related variables */
-	int webpid = 0;
-
-	// NODE that matches the objective search term.
-	Node *nodematch;
-
 	char username[MAX_USERNAME];
 	char searchterm[MAX_PATH];
 	char eventname[MAX_EVENTNAME];
 	char options[MAX_OPTIONS];
 	char *startrecord;
 	int returncode;
+	int webpid = 0;
+	
+	// NODE that matches the objective search term.
+	Node *nodematch;
 
+	DebugMsg("Program starting.\n");
 	/* The only person who can run auditsvc is root, so lets
 	   make sure that we discourage other users */
 	if (getuid() != 0) {
-		fprintf(stderr, "This program can only be run by root.\n");
+		DebugMsg("This program can only be run by root.\n");
 		exit(1);
 	}
 
 	// Read our configuration file, and set up audit.
-	returncode=read_config_file(0);
+	returncode = read_config_file(0);
 
 	if (!hostid) {
 		getfqdn(hostid);
@@ -135,6 +149,7 @@ int main(int argc, char *argv[])
 		webpid = fork();
 		if (webpid == 0) {
 			// Child process - Run our web server
+			DebugMsg("Starting web server.\n");
 			while (InitWebServer
 			       (remote_webport, remote_restrictip,
 				remote_password) == -1) {
@@ -148,15 +163,16 @@ int main(int argc, char *argv[])
 			exit(0);
 		}
 	}
+	DebugMsg("entering loop.\n");
 
-	
-	while(fgets(logbuffer,sizeof(logbuffer),stdin)) {
-		startrecord=logbuffer;
-		username[0]='\0';
-		searchterm[0]='\0';
-		eventname[0]='\0';
-		options[0]='\0';
-		returncode=0;
+	while (fgets(logbuffer, sizeof(logbuffer), stdin)) {
+		startrecord = logbuffer;
+		username[0] = '\0';
+		searchterm[0] = '\0';
+		eventname[0] = '\0';
+		options[0] = '\0';
+		returncode = 0;
+		//if (strcmp(searchterm,"/home/u0000190/SNARE-OUT")) DebugMsg("values: == %s == %s == %s == %s == %d\n", username, searchterm, eventname, options, returncode);
 
 		/* This is where we implement the SNARE event checker process */
 		returncode =
@@ -166,6 +182,7 @@ int main(int argc, char *argv[])
 				  eventname,
 				  options);
 
+		//if (strcmp(searchterm,"/home/u0000190/SNARE-OUT")) DebugMsg("values: == %s == %s == %s == %s == %d\n", username, searchterm, eventname, options, returncode);
 		while ((nodematch =
 			CheckObjective(username,
 				       searchterm,
@@ -179,19 +196,15 @@ int main(int argc, char *argv[])
 			     nodematch->
 			     criticality) != 0) {
 				// Report an error, but continue
-				fprintf(stderr,
-					"error sending log entry to destination\n");
+				DebugMsg("error sending log entry to destination\n");
 			}
 		}
 	}
+	DebugMsg("All done, cleaning up\n");
 
 	if (webpid) {
 		kill(webpid, SIGTERM);
-		waitpid(webpid,NULL,0);
-	}
-
-	if (AuditSocket) {
-		close(AuditSocket);
+		waitpid(webpid, NULL, 0);
 	}
 
 	DestroyList();
@@ -204,9 +217,9 @@ int main(int argc, char *argv[])
 // Shortread: don't re-read all the objectives - just the general configuration items.
 int read_config_file(int shortread)
 {
-	FILE *configfile = (FILE *) NULL;
+	FILE *configfile = (FILE *)NULL;
 	char inputbuffer[MAX_AUDIT_CONFIG_LINE];	// Should be enough for most config lines.
-	// Would love to use gchar here instead, but need to keep this simple.
+							// Would love to use gchar here instead, but need to keep this simple.
 	// Config file header.
 	int headertype = 0;
 
@@ -221,9 +234,9 @@ int read_config_file(int shortread)
 
 	configfile = fopen(CONFIG_FILENAME, "r");
 
-	if (configfile == (FILE *) NULL) {
-		DebugMsg("Cannot open audit configuration file");
-		return (0);
+	if (configfile == (FILE *)NULL) {
+		fprintf(stderr,"Cannot open audit configuration file");
+		return(0);
 	}
 
 	while (fgets(inputbuffer, MAX_AUDIT_CONFIG_LINE, configfile)) {
@@ -252,14 +265,13 @@ int read_config_file(int shortread)
 					int criticality;
 					int returncode;
 					int excludeflag = 0;
-					int excludematchflag=0;
+					int excludematchflag = 0;
 
 					// Do we need to skip over the objectives?
 					if (shortread == 1) {
 						continue;
 					}
 
-
 					if ((criticality =
 					     splitobjective(inputbuffer,
 							    event, user,
@@ -277,10 +289,7 @@ int read_config_file(int shortread)
 
 						// While there are no more commas
 						while (eventpointer2) {
-							eventpointer2 =
-							    strstr
-							    (eventpointer,
-							     ",");
+							eventpointer2 = strstr(eventpointer, ",");
 							if (eventpointer2
 							    ==
 							    (char *) NULL)
@@ -342,7 +351,8 @@ int read_config_file(int shortread)
 								*position =
 								    '\0';
 							}
-
+ 
+							//DebugMsg("About to AddToList...\n");
 							if (!strcmp
 							    (event,
 							     "*"))
@@ -369,7 +379,7 @@ int read_config_file(int shortread)
 							}
 						}
 					} else {
-						fprintf(stderr,
+						DebugMsg(
 							"WARNING: Cannot process objective - please ensure configuration line\n\tcontains valid criticality, event, criticality, return, user and match elements\n\t[%s]\n",
 							inputbuffer);
 					}
@@ -377,14 +387,14 @@ int read_config_file(int shortread)
 					if (isfilename(inputbuffer)) {
 						if (!open_audit_output
 						    (inputbuffer)) {
-							fprintf(stderr,
+							DebugMsg(
 								"WARNING: Could not open the file specified in the audit configuration: %s - Auditing to STDOUT\n",
 								inputbuffer);
 						}
 					} else if (isnetwork(inputbuffer)) {
 						if (!open_audit_network
 						    (inputbuffer)) {
-							fprintf(stderr,
+							DebugMsg(
 								"WARNING: Could not open the network specified in the audit configuration: %s - Auditing to STDOUT\n",
 								inputbuffer);
 						}
@@ -392,7 +402,7 @@ int read_config_file(int shortread)
 						SyslogDestination=get_syslog_dest(inputbuffer);
 						if(!SyslogDestination) {
 							SyslogDestination=13;
-							fprintf(stderr,"WARNING: Could not establish the correct syslog destination from the audit configuration: %s - Sending to user.notice\n",inputbuffer);
+							DebugMsg("WARNING: Could not establish the correct syslog destination from the audit configuration: %s - Sending to user.notice\n",inputbuffer);
 						}
 					}
 
@@ -446,60 +456,55 @@ int read_config_file(int shortread)
 						}
 					}
 				} else {
-					fprintf(stderr,
-						"WARNING: Configuration file line does not fit in any recognised header.\n\t%s\n",
-						inputbuffer);
+					DebugMsg("WARNING: Configuration file line does not fit in any recognised header.\n\t%s\n", inputbuffer);
 				}
 			}
 		}
 	}
 
-	fclose((FILE *) configfile);
-	return (1);
+	fclose((FILE *)configfile);
+	return(1);
 }
 
 int iscomment(char *line)
 {
 	// Verify that there is something to check.
-	if (line == (char *) NULL)
-		return (1);
+	if (line == (char *)NULL) return(1);
 
 	// And that there is some data within.
-	if (strlen(line) == 0 || strlen(line) > MAX_AUDIT_CONFIG_LINE)
-		return (1);
+	if (strlen(line) == 0 || strlen(line) > MAX_AUDIT_CONFIG_LINE) return(1);
 
 	// If the first non-whitespace character is a hash, this is a comment line.
 	while (*line) {
 		// Space or tab or newline / formfeed
-		if (*line == ' ' || *line == 9 || *line == 10
-		    || *line == 12) {
+		if (*line == ' ' || *line == 9 || *line == 10 || *line == 12) {
 			line++;
 		} else if (*line == '#') {
-			return (1);
+			return(1);
 		} else {
 			// Ahh. A non-whitespace, non hash character.
-			return (0);
+			return(0);
 		}
 	}
 
 	// If we are here, then the whole line must have been whitespace
-	return (1);
+	return(1);
 }
 
 int isheader(char *string)
 {
 	if (string[0] == '[' && string[strlen(string) - 1] == ']') {
-		return (1);
+		return(1);
 	}
-	return (0);
+	return(0);
 }
 
 int isfilename(char *string)
 {
 	if (regmatch(string, "^file=")) {
-		return (1);
+		return(1);
 	} else {
-		return (0);
+		return(0);
 	}
 }
 
@@ -507,15 +512,15 @@ int isfilename(char *string)
 int isnetwork(char *string)
 {
 	if (regmatch(string, "^network=")) {
-		return (1);
+		return(1);
 	} else {
-		return (0);
+		return(0);
 	}
 }
 
 int issyslog(char *string)
 {
-	if(regmatch(string,"^syslog=")) {
+	if(regmatch(string, "^syslog=")) {
 		return(1);
 	}
 	return(0);
@@ -527,19 +532,20 @@ int open_audit_output(char *string)
 	if (strlen(string)) {
 		if (regmatchi(string, "stdout")) {
 			AuditDestination |= AUDIT_TO_STDOUT;
-			return (1);
+			return(1);
 		}
 		AuditFile = fopen(string, "a");
+		DebugMsg("Openning log file: %s\n", string);
 		if (AuditFile != (FILE *) NULL) {
 			AuditDestination |= AUDIT_TO_FILE;
-			return (1);
+			return(1);
 		} else {
 			AuditDestination |= AUDIT_TO_STDOUT;
-			return (0);
+			return(0);
 		}
 	}
 
-	return (0);
+	return(0);
 }
 
 // For the moment, require the web server to do the work of converting
@@ -547,9 +553,9 @@ int open_audit_output(char *string)
 int get_syslog_dest(char *string)
 {
 	int destination;
-	string+=strlen("syslog=");
-	destination=atoi(string);
-	if(destination > 0) {
+	string += strlen("syslog=");
+	destination = atoi(string);
+	if (destination > 0) {
 		return(destination);
 	}
 	return(0);
@@ -558,9 +564,10 @@ int get_syslog_dest(char *string)
 // Open the network - destination host and port
 int open_audit_network(char *string)
 {
+	int AuditSocket=0;
 	char desthost[MAX_HOSTID];
-	int port = 6161;	// Default port
-	char *pos;
+	int port = 6161, protocol=SOCKETTYPE_UDP;	// Default port, protocol
+	char *pos, *pos2;
 	struct hostent *hp;
 
 	// Initialise desthost.
@@ -570,15 +577,7 @@ int open_audit_network(char *string)
 	if (strlen(string)) {
 		if (regmatchi(string, "stdout")) {
 			AuditDestination |= AUDIT_TO_STDOUT;
-			return (1);
-		}
-
-		AuditSocket = socket(AF_INET, SOCK_DGRAM, 0);
-		if (AuditSocket < 0) {
-			fprintf(stderr,
-				"Cannot open network socket - auditing to STDOUT\n");
-			AuditDestination |= AUDIT_TO_STDOUT;
-			return (0);
+			return(1);
 		}
 
 		pos = strstr(string, ":");
@@ -593,6 +592,7 @@ int open_audit_network(char *string)
 			pos++;
 			// As long as the user has a number there.
 			if (strlen(pos)) {
+				//check if the protocol is given
 				port = atoi(pos);
 			}
 		} else {
@@ -600,6 +600,13 @@ int open_audit_network(char *string)
 			strncpy(desthost, string, MAX_HOSTID);
 		}
 
+		AuditSocket = socket(AF_INET,SOCK_DGRAM,0);
+		if (AuditSocket < 0) {
+			DebugMsg("Cannot open network socket - auditing to STDOUT\n");
+			AuditDestination |= AUDIT_TO_STDOUT;
+			return(0);
+		}
+
 		// Just in case the user has entered a blank field
 		if (strlen(desthost) == 0) {
 			strncpy(desthost, "localhost", MAX_HOSTID);
@@ -607,26 +614,44 @@ int open_audit_network(char *string)
 
 		hp = gethostbyname(desthost);
 		if (hp == 0) {
-			fprintf(stderr,
-				"Cannot resolve host %s - auditing to STDOUT\n",
-				hostid);
+			DebugMsg("Cannot resolve host %s - auditing to STDOUT\n", hostid);
 			close(AuditSocket);
 			AuditDestination |= AUDIT_TO_STDOUT;
-			return (0);
+			return(0);
 		}
-		bcopy(hp->h_addr, &AuditSocketName.sin_addr, hp->h_length);
-		AuditSocketName.sin_family = AF_INET;
-		AuditSocketName.sin_port = htons(port);
+		hcurrent=(HostNode *)malloc(sizeof(HostNode));
+		if (hcurrent) {
+			bzero((char *) &hcurrent->AuditSocketName, sizeof(hcurrent->AuditSocketName));
+			memcpy(&hcurrent->AuditSocketName.sin_addr, hp->h_addr, hp->h_length);
+			hcurrent->AuditSocketName.sin_family = AF_INET;
+			hcurrent->AuditSocketName.sin_port = htons(port);
+			hcurrent->dest_addr_size=sizeof(hcurrent->AuditSocketName);
+			hcurrent->AuditSocket=AuditSocket;
+			strncpy(hcurrent->desthost, desthost,MAX_HOSTID);
+			hcurrent->port=port;
+			hcurrent->protocol=protocol;
+			hcurrent->next=NULL;
 
-		AuditDestination |= AUDIT_TO_NETWORK;
-		if(port == 514) {
-			AuditDestination |= AUDIT_TO_SYSLOG;
+			HNodeCounter++;
+			hcurrent->cacheflag = 0;
+			hhead=hcurrent;
+			htail=hcurrent;
+			AuditDestination |= AUDIT_TO_NETWORK;
+			if(port == 514) {
+				AuditDestination |= AUDIT_TO_SYSLOG;
+			}
+			return(1);
+		} else {
+			DebugMsg("Cannot allocate memory for host details - auditing to STDOUT\n");
+			close(AuditSocket);
+			AuditDestination |= AUDIT_TO_STDOUT;
+			return(0);
 		}
-		return (1);
+		return(1);
 	}
 
 	AuditDestination |= AUDIT_TO_STDOUT;
-	return (0);
+	return(0);
 }
 
 int getheader(char *string)
@@ -642,21 +667,19 @@ int getheader(char *string)
 	stringp[strlen(stringp) - 1] = '\0';
 
 	if (regmatchi(stringp, "^objectives$")) {
-		return (CONFIG_OBJECTIVES);
+		return(CONFIG_OBJECTIVES);
 	} else if (regmatchi(stringp, "^events$")) {
-		return (CONFIG_EVENTS);
-	} else if (regmatchi(stringp, "^file$")
-		   || regmatchi(stringp, "^output$")) {
-		return (CONFIG_OUTPUT);
+		return(CONFIG_EVENTS);
+	} else if (regmatchi(stringp, "^file$") || regmatchi(stringp, "^output$")) {
+		return(CONFIG_OUTPUT);
 	} else if (regmatchi(stringp, "^hostid$")) {
-		return (CONFIG_HOSTID);
+		return(CONFIG_HOSTID);
 	} else if (regmatchi(stringp, "^remote$")) {
-		return (CONFIG_REMOTE);
+		return(CONFIG_REMOTE);
 	}
 
-	fprintf(stderr, "Unknown header in configuration file: %s\n",
-		stringp);
-	return (0);
+	DebugMsg("Unknown header in configuration file: %s\n", stringp);
+	return(0);
 }
 
 // Return the host identifier
@@ -670,12 +693,12 @@ char *getconfstring(char *string, char *file, int length)
 		if (strlen(stringp)) {
 			strncpy(file, stringp, length - 1);
 		} else {
-			return ((char *) NULL);
+			return((char *)NULL);
 		}
 	} else {
-		return ((char *) NULL);
+		return((char *)NULL);
 	}
-	return ((char *) file);
+	return((char *)file);
 }
 
 int getport(char *string)
@@ -685,16 +708,16 @@ int getport(char *string)
 
 	stringp = strstr(string, "=");
 
-	if (stringp != (char *) NULL) {
+	if (stringp != (char *)NULL) {
 		stringp++;
 		if (strlen(stringp)) {
 			strncpy(strPort, stringp, 10);
-			return (atoi((char *) strPort));
+			return(atoi((char *)strPort));
 		} else {
-			return (0);
+			return(0);
 		}
 	} else {
-		return (0);
+		return(0);
 	}
 }
 
@@ -704,17 +727,17 @@ char *gethostident(char *string, char *host, int length)
 	char *stringp = string;
 
 	stringp = strstr(string, "=");
-	if (stringp != (char *) NULL) {
+	if (stringp != (char *)NULL) {
 		stringp++;
 		if (strlen(stringp)) {
 			strncpy(host, stringp, length - 1);
 		} else {
-			return ((char *) NULL);
+			return((char *)NULL);
 		}
 	} else {
-		return ((char *) NULL);
+		return((char *)NULL);
 	}
-	return ((char *) host);
+	return((char *)host);
 }
 
 // Return a string that contains the criticality
@@ -722,20 +745,19 @@ char *gethostident(char *string, char *host, int length)
 char *criticalitystring(int criticality, char *string)
 {
 	switch (criticality) {
-	case CRITICALITY_CRITICAL:
-		return (strncpy(string, "critical", MAX_AUDITREC));
-	case CRITICALITY_PRIORITY:
-		return (strncpy(string, "priority", MAX_AUDITREC));
-	case CRITICALITY_WARNING:
-		return (strncpy(string, "warning", MAX_AUDITREC));
-	case CRITICALITY_INFO:
-		return (strncpy(string, "information", MAX_AUDITREC));
-	default:
-		return (strncpy(string, "clear", MAX_AUDITREC));
+		case CRITICALITY_CRITICAL:
+			return(strncpy(string, "critical", MAX_AUDITREC));
+		case CRITICALITY_PRIORITY:
+			return(strncpy(string, "priority", MAX_AUDITREC));
+		case CRITICALITY_WARNING:
+			return(strncpy(string, "warning", MAX_AUDITREC));
+		case CRITICALITY_INFO:
+			return(strncpy(string, "information", MAX_AUDITREC));
+		default:
+			return(strncpy(string, "clear", MAX_AUDITREC));
 	}
 }
 
-
 // Remove start / end whitespace, including the newline
 void trim(char *string)
 {
@@ -743,30 +765,25 @@ void trim(char *string)
 	char *pointer2;
 
 	// Verify that there is something to check.
-	if (string == (char *) NULL)
-		return;
+	if (string == (char *) NULL) return;
 
 	// And that there is some data within.
-	if (strlen(string) == 0)
-		return;
+	if (strlen(string) == 0) return;
 
 	// Start from the end, work backwards.
 	pointer = &string[strlen(string) - 1];
 
-	while ((*pointer == ' ' || *pointer == 9 || *pointer == 10
-		|| *pointer == 12) && pointer >= string) {
+	while ((*pointer == ' ' || *pointer == 9 || *pointer == 10 || *pointer == 12) && pointer >= string) {
 		*pointer = '\0';
 		pointer--;
 	}
 
 	// Are we back at the start of the string? If so, this line must be null.
-	if (pointer == string)
-		return;
+	if (pointer == string) return;
 	// Pointer is now at the last non-whitespace character of the string.
 
 	pointer2 = string;
-	while ((*pointer2 == ' ' || *pointer2 == 9 || *pointer2 == 10
-		|| *pointer2 == 12) && pointer2 < pointer) {
+	while ((*pointer2 == ' ' || *pointer2 == 9 || *pointer2 == 10 || *pointer2 == 12) && pointer2 < pointer) {
 		pointer2++;
 	}
 	// pointer2 will now point to the start of the first non-null character
@@ -780,19 +797,16 @@ void trimallwhitespace(char *string)
 	char *writepointer;
 
 	// Verify that there is something to check.
-	if (string == (char *) NULL)
-		return;
+	if (string == (char *) NULL) return;
 
 	// And that there is some data within.
-	if (strlen(string) == 0)
-		return;
+	if (strlen(string) == 0) return;
 
 	// Start from the end, work backwards.
 	readpointer = string;
 	writepointer = string;
 	while (readpointer < (string + strlen(string))) {
-		if (*readpointer == ' ' || *readpointer == 9
-		    || *readpointer == 10 || *readpointer == 12) {
+		if (*readpointer == ' ' || *readpointer == 9 || *readpointer == 10 || *readpointer == 12) {
 			readpointer++;
 		} else {
 			if (writepointer != readpointer) {
@@ -816,16 +830,14 @@ char *strip(char *string)
 		}
 		stringp++;
 	}
-	return (string);
+	return(string);
 }
 
 // Take an identified objective line
 // NOTE: string, event, user and match must all be the same buffer size!
-int splitobjective(char *string, char *event, char *user, char *match,
-		   int *excludeflag, int *excludematchflag, int *returncode)
+int splitobjective(char *string, char *event, char *user, char *match, int *excludeflag, int *excludematchflag, int *returncode)
 {
-	char *startevent, *startuser, *startmatch, *startcrit,
-	    *startreturn;
+	char *startevent, *startuser, *startmatch, *startcrit, *startreturn;
 	char *stringpointer, *endstring;
 	char *eventp, *userp, *matchp, *critp;
 	char criticality[MAX_AUDIT_CONFIG_LINE];
@@ -834,31 +846,28 @@ int splitobjective(char *string, char *event, char *user, char *match,
 	*returncode = RETURNCODE_ANY;
 
 	// Do some basic sanity checks.
-	if (string == (char *) NULL || event == (char *) NULL
-	    || user == (char *) NULL || match == (char *) NULL) {
-		return (-1);
+	if (string == (char *) NULL || event == (char *) NULL || user == (char *) NULL || match == (char *) NULL) {
+		return(-1);
 	}
 	if (!strlen(string)) {
-		return (-1);
+		return(-1);
 	}
 
 	startcrit = strstr(string, "criticality=");
 	startevent = strstr(string, "event=");
 	startreturn = strstr(string, "return=");
 	startuser = strstr(string, "user=");
-	if (startuser == (char *) NULL) {
+	if (startuser == (char *)NULL) {
 		startuser = strstr(string, "user!=");
-		if (startuser != (char *) NULL) {
+		if (startuser != (char *)NULL) {
 			// EXCLUDE USERS rather than include.
 			*excludeflag = 1;
 		}
 	}
 	startmatch = strstr(string, "match=");
-	if (startmatch == (char *) NULL)
-	{
-		startmatch = strstr (string, "match!=");
-		if (startmatch != (char *) NULL)
-		{
+	if (startmatch == (char *)NULL) {
+		startmatch = strstr(string, "match!=");
+		if (startmatch != (char *)NULL) {
 			// EXCLUDE rather than include.
 			*excludematchflag = 1;
 		}
@@ -878,10 +887,8 @@ int splitobjective(char *string, char *event, char *user, char *match,
 	    startmatch == (char *) NULL || startcrit == (char *) NULL
 	    || startreturn == (char *) NULL) {
 		// Problem, this line is malformed. We really cannot proceed with this line.
-		fprintf(stderr,
-			"The following line does not contain the criticality, event, return, user and match elements: [%s]",
-			string);
-		return (-1);
+		DebugMsg("The following line does not contain the criticality, event, return, user and match elements: [%s]", string);
+		return(-1);
 	}
 	// Start with the event
 	stringpointer = startcrit + strlen("criticality=");
@@ -898,10 +905,8 @@ int splitobjective(char *string, char *event, char *user, char *match,
 	*critp = '\0';
 	trim(criticality);
 	crit = atoi(criticality);
-	if (crit < CRITICALITY_CLEAR)
-		crit = CRITICALITY_CLEAR;
-	if (crit > CRITICALITY_CRITICAL)
-		crit = CRITICALITY_CRITICAL;
+	if (crit < CRITICALITY_CLEAR) crit = CRITICALITY_CLEAR;
+	if (crit > CRITICALITY_CRITICAL) crit = CRITICALITY_CRITICAL;
 
 	// Start with the event
 	stringpointer = startevent + strlen("event=");
@@ -950,9 +955,9 @@ int splitobjective(char *string, char *event, char *user, char *match,
 	trim(user);
 
 	if (*excludematchflag) {
-		stringpointer = startmatch + strlen ("match!=");
+		stringpointer = startmatch + strlen("match!=");
 	} else {
-		stringpointer = startmatch + strlen ("match=");
+		stringpointer = startmatch + strlen("match=");
 	}
 
 	while (*stringpointer
@@ -970,10 +975,9 @@ int splitobjective(char *string, char *event, char *user, char *match,
 	trim(match);
 
 	// Return the criticality value.
-	return (crit);
+	return(crit);
 }
 
-
 // Match string against an extended regular expression.
 // Return 1 for match, 0 for no-match or error.
 int regmatch(const char *string, const char *pattern)
@@ -982,14 +986,14 @@ int regmatch(const char *string, const char *pattern)
 	regex_t re;
 
 	if (regcomp(&re, pattern, REG_EXTENDED | REG_NOSUB) != 0) {
-		return (0);
+		return(0);
 	}
 	status = regexec(&re, string, (size_t) 0, NULL, 0);
 	regfree(&re);
 	if (status != 0) {
-		return (0);
+		return(0);
 	}
-	return (1);
+	return(1);
 }
 
 // Match string against an extended regular expression.
@@ -1001,9 +1005,9 @@ int regmatchC(const char *string, regex_t re)
 
 	status = regexec(&re, string, (size_t) 0, NULL, 0);
 	if (status != 0) {
-		return (0);
+		return(0);
 	}
-	return (1);
+	return(1);
 }
 
 // Match string against an extended regular expression. Case insensitive.
@@ -1013,17 +1017,16 @@ int regmatchi(const char *string, const char *pattern)
 	int status;
 	regex_t re;
 
-	if (regcomp(&re, pattern, REG_EXTENDED | REG_NOSUB | REG_ICASE) !=
-	    0) {
-		return (0);
+	if (regcomp(&re, pattern, REG_EXTENDED | REG_NOSUB | REG_ICASE) != 0) {
+		return(0);
 	}
 
 	status = regexec(&re, string, (size_t) 0, NULL, 0);
 	regfree(&re);
 	if (status != 0) {
-		return (0);
+		return(0);
 	}
-	return (1);
+	return(1);
 }
 
 // Linked List Functions
@@ -1047,12 +1050,11 @@ Node *AddToList(char *eventname, char *username, char *path,
 {
 	Node *newNode = NULL;
 
-	newNode = (Node *) malloc(sizeof(Node));
+	newNode = (Node *)malloc(sizeof(Node));
 
 	if (newNode == NULL) {
-		fprintf(stderr,
-			"AddToList(): error in dynamic memory allocation\nCould not add a new objective into our linked list. You may be low on memory.\n");
-		return ((Node *) NULL);
+		DebugMsg("AddToList(): error in dynamic memory allocation\nCould not add a new objective into our linked list. You may be low on memory.\n");
+		return((Node *)NULL);
 	}
 
 	strncpy(newNode->event_name,eventname,MAX_EVENTNAME);
@@ -1069,8 +1071,7 @@ Node *AddToList(char *eventname, char *username, char *path,
 	regcomp(&newNode->pathRE, path, REG_EXTENDED | REG_NOSUB);
 
 	strncpy(newNode->options, options, MAX_OPTIONS);
-	regcomp(&newNode->optionsRE, options,
-		REG_EXTENDED | REG_NOSUB | REG_ICASE);
+	regcomp(&newNode->optionsRE, options, REG_EXTENDED | REG_NOSUB | REG_ICASE);
 
 	if (head == NULL) {
 		head = newNode;
@@ -1101,7 +1102,7 @@ void RemoveFromListHead(void)
 	free(tempPtr);
 }
 
-void RemoveFromList(Node * node)
+void RemoveFromList(Node *node)
 {
 	Node *tempPtr, *previousPtr;
 
@@ -1140,12 +1141,12 @@ void ResetCurrentNode(void)
 
 int IsValidItem(void)
 {
-	return (NULL == currentnode) ? 0 : 1;
+	return(NULL == currentnode) ? 0 : 1;
 }
 
 Node *GetCurrentItem()
 {
-	return (currentnode);
+	return(currentnode);
 }
 
 void NextItemInList(void)
@@ -1173,9 +1174,21 @@ void DestroyList(void)
 
 		free(tempPtr);
 	}
+	hcurrent=hhead;
+	while(hcurrent) {
+		hhead=hcurrent->next;
+		close(hcurrent->AuditSocket);
+		free(hcurrent);
+		hcurrent=hhead;
+	}
+	MCCurrent=MCHead;
+	while(MCCurrent) {
+		MCHead=MCCurrent->next;
+		free(MCCurrent);
+		MCCurrent=MCHead;
+	}
 }
 
-
 // Pull out the fully qualified domain name, if possible.
 char *getfqdn(char *FQDN)
 {
@@ -1184,7 +1197,7 @@ char *getfqdn(char *FQDN)
 
 	if (gethostname(hname, MAX_HOSTID)) {
 		strncpy(FQDN, "localhost.unknown", MAX_HOSTID);
-		return (FQDN);
+		return(FQDN);
 	}
 
 	strncpy(FQDN, hname, MAX_HOSTID);
@@ -1206,35 +1219,69 @@ char *getfqdn(char *FQDN)
 		}
 	}
 
-	return (FQDN);
+	return(FQDN);
 }
 
 int SendToSocket(char *message)
 {
-	if (AuditSocket) {
-		if (sendto(AuditSocket, message,
-			   strlen(message), 0,
-			   (const struct sockaddr *) &AuditSocketName,
-			   sizeof(struct sockaddr))
-		    < 0) {
-			puts(message);
-			return (1);
+	HostNode *myhostcurrent;
+	int bytessent;
+
+	myhostcurrent=hhead;
+	while(myhostcurrent) {
+		if (!myhostcurrent->AuditSocket && !myhostcurrent->last_error) {
+			// The socket is broken, flag an error and let last_error deal with it
+			myhostcurrent->last_error=time(&myhostcurrent->last_error);
 		}
-	} else {
-		puts(message);
-		return (1);
+		if (myhostcurrent->AuditSocket && !myhostcurrent->last_error) {
+			// Send the packet to each of the destinations
+			// defined in the linked list.
+				//DebugMsg("About to send message\n");
+				bytessent = sendto(myhostcurrent->AuditSocket, message, strlen(message), 0, (const struct sockaddr *)&myhostcurrent->AuditSocketName, myhostcurrent->dest_addr_size);
+
+			//DebugMsg("message sent\n");
+			//DebugMsg("Bytes sent to %s: %d\n", myhostcurrent->desthost, bytessent);
+			if (bytessent < 0) {
+				DebugMsg("Error sending to server %s. Retrying in %d seconds\n", myhostcurrent->desthost, WAITTIME);
+				myhostcurrent->last_error=time(&myhostcurrent->last_error);
+			}
+		}
+		myhostcurrent=myhostcurrent->next;
 	}
-	return (0);
+	return(0);
 }
 
+int fix_last_error(HostNode *myhostcurrent) {
+	time_t now;
+	time(&now);
+	if (now - myhostcurrent->last_error >= WAITTIME) {
+		//try to reestablish the connection
+		if(myhostcurrent->AuditSocket) close(myhostcurrent->AuditSocket);
+		myhostcurrent->AuditSocket=0;
+		myhostcurrent->AuditSocket = socket(AF_INET,SOCK_DGRAM,0);
+		if(myhostcurrent->AuditSocket < 0) {
+			// failed to grab a socket, go back to sleep
+			DebugMsg("Cannot open network socket - continuing to wait\n");
+			myhostcurrent->last_error=time(&myhostcurrent->last_error);
+			myhostcurrent->AuditSocket = 0;
+			return(1);
+		}
+		myhostcurrent->last_error = 0;
+	} else {
+		//still waiting
+		return(1);
+	}
+	return(0);
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 // This routine will send an event to the user-selected output device.
 ////////////////////////////////////////////////////////////////////////////////
 int sendevent(char *string, int criticality)
 {
+	static unsigned int EventCounter = 1;
 	char stringout[MAX_AUDITREC] = "";
-	static unsigned int EventCounter=1;
+	char syslogout[MAX_AUDITREC] = "";
 	char blank[2]="";
 	char *position=string;
 	int size=0;
@@ -1249,15 +1296,19 @@ int sendevent(char *string, int criticality)
 	char *startreturn=blank;
 	char *starttrail=blank;
 
+	//DebugMsg("about to send event\n");
 
 	if(!string) {
 		return(0);
 	}
 
 	if(EventCounter >= UINT_MAX) {
-		EventCounter=1;
+		EventCounter = 1;
 	}
 
+	if (AuditDestination & AUDIT_TO_SYSLOG) {
+		strncpy(syslogout,string,MAX_AUDITREC);
+	}
 	size=strlen(string);
 
 	// Work through our string, and convert it into tab-delimited fields.
@@ -1350,6 +1401,7 @@ int sendevent(char *string, int criticality)
 
 	if (AuditDestination & AUDIT_TO_FILE) {
 		// Write to file
+		//DebugMsg("writing to file\n");
 		if (AuditFile != (FILE *) NULL) {
 			fprintf(AuditFile, "%s\n", stringout);
 			fflush(AuditFile);
@@ -1357,28 +1409,25 @@ int sendevent(char *string, int criticality)
 	}
 
 	if (AuditDestination & AUDIT_TO_NETWORK) {
-		if(AuditDestination & AUDIT_TO_SYSLOG) {
+		//DebugMsg("writing to network\n");
+		if (AuditDestination & AUDIT_TO_SYSLOG) {
 			// Do Syslog stuff here.
 			time_t currenttime;
 			struct tm *newtime;
-			char CurrentDate[16]="";
+			char CurrentDate[16] = "";
 
 			time(&currenttime);
-			newtime=localtime(&currenttime);
-			syslogdate(CurrentDate,newtime);
-
-			snprintf(stringout,MAX_AUDITREC,"<%d> %s %s AIXAudit\t%d\t%s",SyslogDestination,CurrentDate,hostid,criticality,string);
-		}
-		if (AuditSocket) {
-			SendToSocket(stringout);
+			newtime = localtime(&currenttime);
+			syslogdate(CurrentDate, newtime);
+			snprintf(stringout, MAX_AUDITREC, "<%d> %s %s AIXAudit\t%d\t%s",SyslogDestination,CurrentDate,hostid,criticality,syslogout);
 		}
+		SendToSocket(stringout);
 	}
 
-	return (0);
+	return(0);
 }
 
-Node *CheckObjective(char *username, char *searchterm, char *eventname,
-		     char *options, int returncode)
+Node *CheckObjective(char *username, char *searchterm, char *eventname, char *options, int returncode)
 {
 	static int firstcall = 0;
 	static Node *currentnode;
@@ -1395,13 +1444,13 @@ Node *CheckObjective(char *username, char *searchterm, char *eventname,
 		    && (returncode == currentnode->returncode
 			|| currentnode->returncode == RETURNCODE_ANY)) {
 			// Are we including users, or excluding.
-			if (currentnode->excludeflag ^ regmatchC (username, currentnode->usernameRE)) {
-				if (currentnode->excludematchflag ^ regmatchC (searchterm, currentnode->pathRE)) {
+			if (currentnode->excludeflag ^ regmatchC(username, currentnode->usernameRE)) {
+				if (currentnode->excludematchflag ^ regmatchC(searchterm, currentnode->pathRE)) {
 					if (regmatchC
 					    (options,
 					     currentnode->optionsRE)) {
 						NextItemInList();
-						return (currentnode);
+						return(currentnode);
 					}
 				}
 			} else {
@@ -1413,7 +1462,7 @@ Node *CheckObjective(char *username, char *searchterm, char *eventname,
 					    (options,
 					     currentnode->optionsRE)) {
 						NextItemInList();
-						return (currentnode);
+						return(currentnode);
 					}
 				}
 			}
@@ -1422,12 +1471,11 @@ Node *CheckObjective(char *username, char *searchterm, char *eventname,
 	}
 
 	ResetCurrentNode();
-	return ((Node *) NULL);
+	return((Node *) NULL);
 }
 
 // Pick out the core details from an audit event, and set the username/searchterm/eventname
-int GetAIXDetails(char *logbuffer, char *username, char *searchterm,
-		  char *eventname, char *options)
+int GetAIXDetails(char *logbuffer, char *username, char *searchterm, char *eventname, char *options)
 {
 	char *position = logbuffer;
 	char tempreturn[20]="";
@@ -1456,16 +1504,19 @@ int GetAIXDetails(char *logbuffer, char *username, char *searchterm,
 	// Grab the date first.
 	if(eventsize > 16) { eventsize=16; }
 	strncpy(eventname,position,eventsize);
+	eventname[eventsize]='\0';
 
 	// Kill whitespace
 	trim(eventname);
 	position=logbuffer+73;
 	if(usersize>9) { usersize=9; }
 	strncpy(username,position,usersize);
+	username[usersize]='\0';
 	trim(username);
 
 	position=logbuffer+109;
 	strncpy(tempreturn,position,20);
+	
 	trim(tempreturn);
 
 	if(!strcmp(eventname,"PROC_Execute") || !strcmp(eventname,"PROC_LPExecute")) {
@@ -1512,7 +1563,7 @@ int GetAIXDetails(char *logbuffer, char *username, char *searchterm,
 				// Indicates read vs write.
 				// Copy until the next space.
 				strncpy(options,position,optionssize);
-				options[optionssize]='\0';
+				options[optionssize-1]='\0';
 			}
 		}
 	}
@@ -1527,23 +1578,24 @@ int GetAIXDetails(char *logbuffer, char *username, char *searchterm,
 			strncpy(searchterm,logbuffer,searchsize);
 		}
 	}
+	//searchterm[searchsize]='\0';
 
 	if (!strcmp(tempreturn, "OK")) {
-		return (RETURNCODE_SUCCESS);
+		return(RETURNCODE_SUCCESS);
 	}
-	return (RETURNCODE_FAILURE);
+	return(RETURNCODE_FAILURE);
 }
 
 char *FindToken(char *token, char *buffer)
 {
 	char *position;
 
-	if(!token || !buffer) {
+	if (!token || !buffer) {
 		return((char *)NULL);
 	}
-	position=strstr(buffer, token);
-	if(position) {
-		position +=strlen(token);
+	position = strstr(buffer, token);
+	if (position) {
+		position += strlen(token);
 	}
 	return(position);
 }
@@ -1556,24 +1608,24 @@ void syslogdate(char *sdate, struct tm *cdate)
 	char Min[3];
 	char Sec[3];
 
-	if(!sdate || !cdate) return;
+	if (!sdate || !cdate) return;
 
 	switch (cdate->tm_mon) {
-		case 0: strcpy(Month,"Jan"); break;
-		case 1: strcpy(Month,"Feb"); break;
-		case 2: strcpy(Month,"Mar"); break;
-		case 3: strcpy(Month,"Apr"); break;
-		case 4: strcpy(Month,"May"); break;
-		case 5: strcpy(Month,"Jun"); break;
-		case 6: strcpy(Month,"Jul"); break;
-		case 7: strcpy(Month,"Aug"); break;
-		case 8: strcpy(Month,"Sep"); break;
-		case 9: strcpy(Month,"Oct"); break;
-		case 10: strcpy(Month,"Nov"); break;
-		default: strcpy(Month,"Dec"); break;
+		case 0: strcpy(Month, "Jan"); break;
+		case 1: strcpy(Month, "Feb"); break;
+		case 2: strcpy(Month, "Mar"); break;
+		case 3: strcpy(Month, "Apr"); break;
+		case 4: strcpy(Month, "May"); break;
+		case 5: strcpy(Month, "Jun"); break;
+		case 6: strcpy(Month, "Jul"); break;
+		case 7: strcpy(Month, "Aug"); break;
+		case 8: strcpy(Month, "Sep"); break;
+		case 9: strcpy(Month, "Oct"); break;
+		case 10: strcpy(Month, "Nov"); break;
+		default: strcpy(Month, "Dec"); break;
 	}
 
-	if(cdate->tm_mday<10) {
+	if (cdate->tm_mday < 10) {
 		snprintf(Date,3," %d%c",cdate->tm_mday,0);
 	} else {
 		snprintf(Date,3,"%d%c",cdate->tm_mday,0);
@@ -1600,22 +1652,43 @@ void syslogdate(char *sdate, struct tm *cdate)
 	snprintf(sdate,16,"%s %s %s:%s:%s%c",Month,Date,Hour,Min,Sec,0);
 }
 
+int IsSocketValid(int sock) {
+	struct fd_set connectionSet;
+	struct timeval timeout;
+	int result;
+
+	//set timeout values
+	timeout.tv_sec =0;
+	timeout.tv_usec =250;
+
+	FD_ZERO(&connectionSet);
+	FD_SET(sock, &connectionSet);
+
+	//call select to see if data is waiting on the socket
+	result = select(sock+1, &connectionSet, NULL, NULL, &timeout);
+
+	//check results
+	//DebugMsg("SELECT result: %d\n", result);
+	return (result == 0);
+}
 
 void DebugMsg(const char *pszFormat, ...)
 {
 	char buf[8192] = "";
-	// FILE *fp;
+	FILE *fp;
 
 	va_list arglist;
 	va_start(arglist, pszFormat);
-	vsnprintf(&buf[strlen(buf)], 8192 - strlen(buf) - 1, pszFormat,
-		  arglist);
+	vsnprintf(&buf[strlen(buf)], 8192 - strlen(buf) - 1, pszFormat, arglist);
 	va_end(arglist);
 
-	printf("%s\n",buf);
-	fflush(stdout);
-
-	//fp = fopen("/tmp/SNARE-OUT", "a");
-	//fprintf(fp,"%s\n",buf);
-	//fclose(fp);
+	fp=fopen("/home/u0000190/SNARE-OUT", "a");
+	if (fp) {
+		fputs(buf,fp);
+		fclose(fp);
+	} else {
+		printf("\nERROR: There was an error opening the log file\n");
+	}
+	//printf("%s\n", buf);
+	//fflush(stdout);
 }
